# -*- coding: utf-8 -*-
# figur.py

from grafikfenster import *
from math import radians

from abc import * # abstrakte Klasse realisieren

import platform
versionsNummer=platform.python_version()[0]
if versionsNummer=='2': from abstrakt2 import Abstrakt
else: from abstrakt3 import Abstrakt
# Hilfsfunktionen
first = lambda p: p[0]
second = lambda p: p[1]

### ----- Figur --------------------------------------------
class Figur(Abstrakt):
    """Abstrakte Oberklasse fuer alle Figuren
    @author Claus Albowski
    @version November 18"""

    def __init__(self, x=0, y=0,
                 breite=1, hoehe=1,
                 winkel=0, farbe='black', fuellfarbe=None):
        """Konstruktor"""
        self.__x, self.__y, self.__winkel = x,y,winkel
        self.__breite, self.__hoehe = breite,hoehe
        self.__farbe = farbe
        self.__fuellfarbe = fuellfarbe
        self.__sichtbar = True
        self.__gruppe = None
        self.Zeige()
        
    @abstractmethod
    def GibFigur(self):
        """definiert die zu zeichnende Figur (Pfad)"""
        return NotImplemented

    ## Erweiterungen wegen der Textobjekte:
    def IstText(self):
        '''Zur eindeutigen Identifikation von Textobjekten'''
        return False

    def Transformiere(self, path):
        """Transformiert den übergebenen Pfad"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        gc.PushState()
        # Hilfvariable verwenden
        x,y = self.GibXPosition(), self.GibYPosition()
        w = self.GibOrientierung()
        b,h = self.GibBreite(), self.GibHoehe()
        gc.Translate(x+b/2, y+h/2)
        gc.Rotate(radians(w))
        gc.Translate(-b/2, -h/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        if self.__gruppe != None:
            return self.__gruppe.Transformiere(path)
        return path

    def SetzeGruppe(self, gruppe):
        self.__gruppe = gruppe

    def GibXPosition(self):
        """Get-Methode fuer die xPosition"""
        return self.__x
    
    def GibYPosition(self):
        """Get-Methode fuer die yPosition"""
        return self.__y
    
    def GibBreite(self):
        """Get-Methode fuer die Breite (Klassenvariable)"""
        return self.__breite
    
    def GibHoehe(self):
        """Get-Methode fuer die Hoehe"""
        return self.__hoehe
    
    def GibOrientierung(self):
        """Get-Methode fuer die Orientierung"""
        return self.__winkel
    
    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        return self.__farbe
    
    def GibFuellFarbe(self):
        """Get-Methode fuer die Fuellfarbe
        gibt None zurueck, wenn nur der Umriss gezeichnet
        werden soll"""
        return self.__fuellfarbe
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.__sichtbar

    def Bewege(self, umX=0, umY=0):
        """Veraendernde Methode fuer die x- und y-Position"""
        self.Verberge()
        self.__x += umX
        self.__y += umY
        self.Zeige()
    
    def AendereAbmessungen(self, neueBreite, neueHoehe):
        """Veraendernde Methode fuer die Breite und Hoehe"""
        self.Verberge()
        self.__breite = neueBreite
        self.__hoehe = neueHoehe
        self.Zeige()
    
    def Skaliere(self, skalierungsFaktor):
        """Veraendernde Methode Breite und Hoehe mit gleichem Faktor"""
        self.Verberge()
        self.__breite *= skalierungsFaktor
        self.__hoehe *= skalierungsFaktor
        self.Zeige()
    
    def Drehe(self, umWinkel=0):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Verberge()
        self.__winkel += umWinkel
        self.Zeige()

    def _SetzeWinkel(self, winkel=0):
        """Veraendernde Methode fuer die Orientierung [Winkel]
        ohne Aenderung der Darstellung"""
        self.__winkel = winkel

    def AendereFarbe(self, neueFarbe='black'):
        """Veraendernde Methode fuer die Farbe
        Standard ist black"""
        self.Verberge()
        self.__farbe = neueFarbe
        self.Zeige()
    
    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Veraendernde Methode fuer die Fuellfarbe
        ohne Parameter wird die Fuellung geloescht."""
        self.Verberge()
        self.__fuellfarbe = neueFuellfarbe
        self.Zeige()
    
    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__sichtbar = False
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__sichtbar = True
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ----- Linie -------------------------------------------- ^ Figur v N_Eck
class Linie(Figur):
    """Klasse Linie ermoeglicht das Zeichnen
    und Bearbeiten einer Linie"""

    def __init__(self,
                 xPos=0,
                 yPos=0,
                 breite=0,
                 hoehe=0,
                 winkel=0,
                 farbe="black"):
        Figur.__init__(self, xPos, yPos, breite, hoehe, winkel, farbe, None)

    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Linien haben keine Fuellfarbe"""
        pass
        
    def GibFigur(self):
        """Definiert den Pfad und transformiert ihn."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        
        # lokale Variable zur Schreibvereinfachung
        b, h = self.GibBreite(), self.GibHoehe()
        path.MoveToPoint(0, 0)
        path.AddLineToPoint(b, h)
        
        return self.Transformiere(path)
    
### ----- N_Eck -------------------------------------------- ^ Linie v FigurenGruppe
class N_Eck(Figur):
    """Klasse N_Eck ermoeglicht das Zeichnen
    und Bearbeiten eines n-Ecks"""

    def __init__(self,
                 xPos=100,
                 yPos=100,
                 ecken=((0,0),(20,10),(0,20)),
                 winkel=0,
                 farbe="black",
                 fuellfarbe=None,
                 geschlossen=True):
        self.__ecken=ecken
        breite = max(map(first,ecken)) - min(map(first,ecken))
        hoehe = max(map(second,ecken)) - min(map(second,ecken))
        self.__geschlossen=geschlossen
        Figur.__init__(self, xPos, yPos, breite, hoehe, winkel, farbe, fuellfarbe)

    def GibFigur(self):
        """Definiert den Pfad."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        
        # lokale Variable zur Schreibvereinfachung
        b, h = self.GibBreite(), self.GibHoehe()
        path.MoveToPoint(self.__ecken[0])
        for i in range(1,len(self.__ecken)):
            path.AddLineToPoint(self.__ecken[i])
        if self.__geschlossen: path.CloseSubpath()
        
        return self.Transformiere(path)

### ----- figurenGruppe ------------------------------------ ^ N_Eck v FigurenGruppe2 
class FigurenGruppe(Figur):
    """Klasse FigurenGruppe ermoeglicht das Zeichnen
    und Bearbeiten von FigurenGruppen.
    Die Kindobjekte werden mit relativer Position eingebaut."""

    def __init__(self,
                 xPos=0,
                 yPos=0,
                 winkel=0):
        self.__figuren = []
        Figur.__init__(self, xPos, yPos, 1, 1, winkel, 'black')
        ## die Figurengruppe hat keine eigene Sichtbarkeit
        
    def AendereAbmessungen(self, neueBreite, neueHoehe):
        """Veraendernde Methode fuer die Breite und Hoehe
        Die relativen Positionen werden korrigiert."""
        faktorX,faktorY = 1.0,1.0
        if self.GibBreite()>0:
            faktorX = float(neueBreite)/self.GibBreite()
        if self.GibHoehe()>0:
            faktorY = float(neueHoehe)/self.GibHoehe()
        self._Figur__breite = neueBreite
        self._Figur__hoehe = neueHoehe
        for figur in self.__figuren:
            b = faktorX * figur.GibBreite()
            h = faktorY * figur.GibHoehe()
            figur.AendereAbmessungen(b, h)
            dx = figur.GibXPosition() * (faktorX - 1.0)
            dy = figur.GibYPosition() * (faktorY - 1.0)
            figur.Bewege(dx, dy)
        
    def AendereFarbe(self, neueFarbe='black'):
        """Gruppen sollten ggf. keine eigene Farbe haben"""
        for figur in self.__figuren:
            figur.AendereFarbe(neueFarbe)
        
    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Gruppen sollten ggf. keine eigene Fuellfarbe haben"""
        for figur in self.__figuren:
            figur.AendereFuellfarbe(neueFuellfarbe)
        
    def GibFigur(self):
        """Definiert den Pfad."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        return path

    def __AktualisiereAbmessungen(self):
        """Interne Methode zum Aktualisieren der Abmessungen
        nach Einbau oder Entfernen einer neuen Figur"""
        b,h = 1,1
        for figur in self.__figuren:
            b=max(figur.GibXPosition()+figur.GibBreite(), b)
            h=max(figur.GibYPosition()+figur.GibHoehe(), h)
        self._Figur__breite,self._Figur__hoehe=b,h

    def FuegeHinzu(self, figur):
        """fuegt der Gruppe eine Figur hinzu
        Voraussetzung fuer richtige Werte ist winkel=0 bei Kindern"""
        self.Verberge()
        if figur not in self.__figuren:
            self.__figuren.append(figur)
            figur.SetzeGruppe(self)
            self.__AktualisiereAbmessungen()
        self.Zeige()
    
    def Entferne(self, figur):
        """entfernt eine Figur aus der Gruppe"""
        self.Verberge()
        if figur in self.__figuren:
            self.__figuren.remove(figur)
            figur.SetzeGruppe(None)
            self.__AktualisiereAbmessungen()
        self.Zeige()
    
    def AendereReihenfolge(self, von, nach):
        """aendert die Reihenfolge, in der die Figuren
        der Gruppe dargestellt werden"""
        anz=len(self.__figuren)
        if (von<1) or (von>anz) or (nach<1) or (nach>anz) or (von==nach): return
        self.Verberge()
        tausch=self.__figuren[von-1]
        self.__figuren[von-1]=self.__figuren[nach-1]
        self.__figuren[nach-1]=tausch
        self.Zeige()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        for figur in self.__figuren:
            figur.Verberge()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        for figur in self.__figuren:
            figur.Zeige()

### ----- FigurenGruppe2 ----------------------------------- ^ FigurenGruppe v Rechteck 
class FigurenGruppe2(FigurenGruppe):
    """FigurenGruppe mit Drehung um den Anfangspunkt"""

    def __init__(self, xPos=0, yPos=0, winkel=0):
        FigurenGruppe.__init__(self, xPos, yPos, winkel)

    def Transformiere(self, path):
        """Transformiert den übergebenen Pfad
        Drehung um den Anfang!!! """
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        gc.PushState()
        # Hilfvariable verwenden
        x,y = self.GibXPosition(), self.GibYPosition()
        w = self.GibOrientierung()
        b,h = self.GibBreite(), self.GibHoehe()
        gc.Translate(x, y)
        gc.Rotate(radians(w))
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        return path

### ----- Rechteck ----------------------------------------- ^ FigurenGruppe2 v Ellipse 
class Rechteck(Figur):
    """Klasse Rechteck ermoeglicht das Zeichnen
    und Bearbeiten eines Rechtecks"""

    def __init__(self,
                 xPos=0,
                 yPos=0,
                 breite=200,
                 hoehe=80,
                 winkel=0,
                 farbe="BLACK",
                 fuellfarbe = "WHITE"):
        Figur.__init__(self, xPos, yPos, breite, hoehe, winkel, farbe, fuellfarbe)
        
    def GibFigur(self):
        """Definiert den Pfad und transformiert ihn."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        
        # lokale Variable zur Schreibvereinfachung
        b, t = self.GibBreite(), self.GibHoehe()
        path.AddRectangle(0, 0, b, t)
        
        return self.Transformiere(path)
    
### ----- Ellipse ------------------------------------------ ^ Rechteck v Bogen
class Ellipse(Figur):
    """Klasse Ellipse ermoeglicht das Zeichnen
    und Bearbeiten einer Ellipse"""

    def __init__(self,
                 xPos=0,
                 yPos=0,
                 breite=200,
                 hoehe=80,
                 winkel=0,
                 farbe="black",
                 fuellfarbe=None):
        Figur.__init__(self, xPos, yPos, breite, hoehe, winkel, farbe, fuellfarbe)
        
    def GibFigur(self):
        """Definiert den Pfad und transformiert ihn."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        
        # lokale Variable zur Schreibvereinfachung
        b, h = self.GibBreite(), self.GibHoehe()
        path.AddEllipse(0, 0, b, h)
        
        return self.Transformiere(path)
    
### ----- Bogen -------------------------------------------- ^ Ellipse v Text
class Bogen(Figur):
    """Die Klasse Bogen ermoeglicht das Zeichnen
    und Bearbeiten eines Bogens"""

    def __init__(self,
                 mitteX=100,
                 mitteY=100,
                 radius=40,
                 winkelStart=100,
                 winkelEnde=330,
                 farbe="black",
                 fuellfarbe=None):
        self.__mX, self.__mY, self.__r  = mitteX, mitteY, radius
        self.__grundfigur = None
        # Die winkel muessen auf Bogenmass umgerechnet werden
        self.__wS, self.__wE = winkelStart*pi/180, winkelEnde*pi/180
        Figur.__init__(self, self.__mX, self.__mY, 0, 0, 0, farbe, fuellfarbe)

    def _SetzeGrundfigur(self):
        """Definiert den Pfad ohne Transformation"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        #AddArc(wxDouble x, wxDouble y, wxDouble r,
        # wxDouble startAngle, wxDouble endAngle, bool clockwise)
        path.AddArc(0, 0, self.__r, self.__wS, self.__wE, True)
        box=path.GetBox()
        # Fuer die Transformation:
        self._Figur__breite = box.width
        self._Figur__hoehe = box.height
        return path
        
    def GibFigur(self):
        """Uebergibt den Pfad der Transformation"""
        if self.__grundfigur == None:
            self.__grundfigur = self._SetzeGrundfigur()
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()
        path.AddPath(self.__grundfigur)
        return self.Transformiere(path)
    
### ----- Text --------------------------------------------- ^ Bogen v ----
class Text(Figur):
    """Klasse Text ermoeglicht das Zeichnen und Bearbeiten von Text"""

    def __init__(self,
                 text="",
                 xPos=0,
                 yPos=0,
                 winkel=0,
                 farbe="black"):
        self.__text = text
        Figur.__init__(self, xPos, yPos, len(text)*10, 20, winkel, farbe, None)
        self.SetzeFont()

    def IstText(self):
        '''Zur eindeutigen Identifikation von Textobjekten'''
        return True

    def GibFont(self):
        '''gibt das von SetzeFont gesetzte font-Objekt zurueck, sonst None'''
        return self.__font

    def SetzeFont(self,
                  pointSize=10,
                  family="DEFAULT",
                  style="NORMAL",
                  weight="FETT",
                  underline=False):
        '''pointSize: 10 ...
        family: "DEFAULT","ROMAN","SWISS"
        style: "NORMAL","KURSIV"
        weight: "NORMAL","FETT"'''
        self.Verberge()
        ff={"DEFAULT":wx.FONTFAMILY_DEFAULT,
            "ROMAN":wx.FONTFAMILY_ROMAN,
            "SWISS":wx.FONTFAMILY_SWISS}
        if not family in ff: family="DEFAULT"
        st={"NORMAL":wx.FONTSTYLE_NORMAL,"KURSIV":wx.FONTSTYLE_ITALIC}
        if not style in st: style="NORMAL"
        w={"NORMAL":wx.FONTWEIGHT_NORMAL ,"FETT":wx.FONTWEIGHT_BOLD}
        if not weight in w: weight="NORMAL"
        self.__font=wx.Font(pointSize,
                            ff[family],
                            st[style],
                            w[weight],
                            underline)
        ##self.__font=wx.Font(20, wx.ROMAN, wx.ITALIC, wx.BOLD, True)
        self.Zeige()
    
    def AendereFuellfarbe(self, neueFuellfarbe=None):
        """Texte haben keine Fuellfarbe"""
        pass
    
    def SetzeGruppe(self, gruppe):
        """Gruppe geht nicht"""
        self.__gruppe = None

    def AendereText(self, text):
        self.Verberge()
        self.__text = text
        self.Zeige()

    def GibText(self):
        return self.__text
        
    def GibFigur(self):
        """Definiert den Pfad und transformiert ihn."""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        path = gc.CreatePath()        
        # sinnlos, daher:        
        return self.Transformiere(path)

### ----- Ende Text ----------------------------------------
### Versuch "underline" hin zu bekommen,
### warum auch immer es nicht geht
### ---------------------------------------------------
class TextTestApp(wx.App):
    """Test-Anwendung speziell fuer Tisch"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung:"""
        global text
        text=Text('mein schoener langer Text',10,100)
        text.SetzeFont(20, wx.ROMAN, wx.ITALIC, wx.BOLD, True)
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = TextTestApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
